; Interrupt test routine - V1.0
;
; This sets up an timer interrupt, driven by the internal clock, 
; and the prescaler set to 64:1. The interrupt routine increases
; a two-byte time value, and outputs the high byte to port B. The
; effect of this is that a Seven Segment LED Plugin connected to
; port b, in 'two digit mode' (a config of 'd2') will count steadily
; up in hex. The exact speed of the count depends on the emulated 
; speed of the microcontroller.
;
;
		list	p=16C84,r=hex

		w		equ	0
		f		equ	1 
		pcl		equ	0x02

		tmr0		equ	0x01
		opt		equ	0x01
		status		equ	0x03
		porta		equ	0x05
		portb		equ	0x06
		trisa		equ 	0x05
		trisb		equ	0x06
		indf		equ	0x00
		fsr		equ	0x04
		eedata		equ	0x08
		eeadr		equ	0x09
		eecon1		equ	0x08
		intcon		equ	0x0b

		rp0		equ 	5	; (Status) Bank select

		intedg		equ	6	; (Option) RB0/Int edge select
		t0cs		equ	5	; (Option) Tmr 0 clock select

		t0if		equ	2	; (Intcon) Tmr 0 overflow flag

		; Start of our data
		w_temp		equ	0x0C	; Interrupt state store
		status_temp	equ	0x0D	; Interrupt state store

		time_l		equ	0x0E	; our counter
		time_h		equ 	0x0F

		org 0x0000
		goto start

		org 0x0004	
		; Interrupt routine - first store status
		movwf		w_temp		; Copy W to W_TEMP register
		swapf		status,w	; Swap status to be saved into W
		movwf		status_temp	; Save status to STATUS_TEMP register

		; Start of our interrupt routine
		bcf		intcon,t0if	; Clear Timer 0 interrupt..

		bcf		status,rp0	; bank 0
		incfsz		time_l		; increment time
		goto 		not_yet
		incf		time_h
		movfw		time_h
		movwf		portb		; output it on port B	

		; End of interrupt - restore status
not_yet		swapf		status_temp,w	; Swap nibbles in STATUS_TEMP register into W
		movwf		status		; Move W into STATUS register
		swapf		w_temp,f	; Swap nibbles in W_TEMP
		swapf		w_temp,w	; Swap nibbles in W_TEMP into W
		retfie

;
; Main program initialisation. Set Port B as output, port A as input and
; interrupt on internal timer with 64:1 prescaler.
;
start		bsf		status,rp0	; bank 1
		movlw		0x00		; All outputs on Port B
		movwf		trisb		
		movlw		0x1f		; All inputs on Port A
                movwf           trisa
		movlw		0xA0		; Global ints and timer overflow
		movwf		intcon
		movlw		0x04		; Timer clock source internal, prescaler = 64
		movwf		opt
		bcf 		status,rp0	; bank 0
		movlw		0xFE
		movwf		tmr0		; Not long to wait..

;
; An empty loop that will be run whilst we're waiting for interrupts.
;
loop
		nop			
		nop		
		nop	
		nop
		nop
		nop	
		goto		loop
